#include <assert.h>

#include <aterm2.h>
#include <deprecated.h>
#include "ADT.h"

/*{{{  typedefs */

typedef struct ATerm _ADTEntries;
typedef struct ATerm _ADTEntry;

/*}}}  */

/*{{{  void ADTinitADTApi(void) */

void ADTinitADTApi(void)
{
  init_ADT_dict();
}

/*}}}  */

/*{{{  term conversion functions */

/*{{{  ADTEntries ADTEntriesFromTerm(ATerm t) */

ADTEntries ADTEntriesFromTerm(ATerm t)
{
  return (ADTEntries)t;
}

/*}}}  */
/*{{{  ATerm ADTEntriesToTerm(ADTEntries arg) */

ATerm ADTEntriesToTerm(ADTEntries arg)
{
  return (ATerm)arg;
}

/*}}}  */
/*{{{  ADTEntry ADTEntryFromTerm(ATerm t) */

ADTEntry ADTEntryFromTerm(ATerm t)
{
  return (ADTEntry)t;
}

/*}}}  */
/*{{{  ATerm ADTEntryToTerm(ADTEntry arg) */

ATerm ADTEntryToTerm(ADTEntry arg)
{
  return (ATerm)arg;
}

/*}}}  */

/*}}}  */
/*{{{  constructors */

/*{{{  ADTEntries ADTmakeEntriesList(ADTEntry head, ADTEntries tail) */

ADTEntries ADTmakeEntriesList(ADTEntry head, ADTEntries tail)
{
  return (ADTEntries)(ATerm)ATinsert((ATermList)tail, (ATerm)head);
}

/*}}}  */
/*{{{  ADTEntry ADTmakeEntryDefault(ADTSort sort, ADTAlternative alt, ADTPattern pat) */

ADTEntry ADTmakeEntryDefault(ADTSort sort, ADTAlternative alt, ADTPattern pat)
{
  return (ADTEntry)(ATerm)ATinsert(ATinsert(ATmakeList1((ATerm)pat), (ATerm)alt), (ATerm)sort);
}

/*}}}  */

/*}}}  */
/*{{{  equality functions */

ATbool ADTisEqualEntries(ADTEntries arg0, ADTEntries arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

ATbool ADTisEqualEntry(ADTEntry arg0, ADTEntry arg1)
{
  return ATisEqual((ATerm)arg0, (ATerm)arg1);
}

/*}}}  */
/*{{{  ADTEntries accessors */

/*{{{  ATbool ADTisValidEntries(ADTEntries arg) */

ATbool ADTisValidEntries(ADTEntries arg)
{
  if (ADTisEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADTisEntriesList(ADTEntries arg) */

inline ATbool ADTisEntriesList(ADTEntries arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADTpatternEntriesList, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADThasEntriesHead(ADTEntries arg) */

ATbool ADThasEntriesHead(ADTEntries arg)
{
  if (ADTisEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADTEntry ADTgetEntriesHead(ADTEntries arg) */

ADTEntry ADTgetEntriesHead(ADTEntries arg)
{
  
    return (ADTEntry)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADTEntries ADTsetEntriesHead(ADTEntries arg, ADTEntry head) */

ADTEntries ADTsetEntriesHead(ADTEntries arg, ADTEntry head)
{
  if (ADTisEntriesList(arg)) {
    return (ADTEntries)ATreplace((ATermList)arg, (ATerm)head, 0);
  }

  ATabort("Entries has no Head: %t\n", arg);
  return (ADTEntries)NULL;
}

/*}}}  */
/*{{{  ATbool ADThasEntriesTail(ADTEntries arg) */

ATbool ADThasEntriesTail(ADTEntries arg)
{
  if (ADTisEntriesList(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADTEntries ADTgetEntriesTail(ADTEntries arg) */

ADTEntries ADTgetEntriesTail(ADTEntries arg)
{
  
    return (ADTEntries)ATgetNext((ATermList)arg);
}

/*}}}  */
/*{{{  ADTEntries ADTsetEntriesTail(ADTEntries arg, ADTEntries tail) */

ADTEntries ADTsetEntriesTail(ADTEntries arg, ADTEntries tail)
{
  if (ADTisEntriesList(arg)) {
    return (ADTEntries)ATreplaceTail((ATermList)arg, (ATermList)tail, 1);
  }

  ATabort("Entries has no Tail: %t\n", arg);
  return (ADTEntries)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  ADTEntry accessors */

/*{{{  ATbool ADTisValidEntry(ADTEntry arg) */

ATbool ADTisValidEntry(ADTEntry arg)
{
  if (ADTisEntryDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  inline ATbool ADTisEntryDefault(ADTEntry arg) */

inline ATbool ADTisEntryDefault(ADTEntry arg)
{
  if (ATisEmpty((ATermList)arg)) {
    return ATfalse;
  }
#ifndef DISABLE_DYNAMIC_CHECKING
  assert(arg != NULL);
  assert(ATmatchTerm((ATerm)arg, ADTpatternEntryDefault, NULL, NULL, NULL));
#endif
  return ATtrue;
}

/*}}}  */
/*{{{  ATbool ADThasEntrySort(ADTEntry arg) */

ATbool ADThasEntrySort(ADTEntry arg)
{
  if (ADTisEntryDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADTSort ADTgetEntrySort(ADTEntry arg) */

ADTSort ADTgetEntrySort(ADTEntry arg)
{
  
    return (ADTSort)ATgetFirst((ATermList)arg);
}

/*}}}  */
/*{{{  ADTEntry ADTsetEntrySort(ADTEntry arg, ADTSort sort) */

ADTEntry ADTsetEntrySort(ADTEntry arg, ADTSort sort)
{
  if (ADTisEntryDefault(arg)) {
    return (ADTEntry)ATreplace((ATermList)arg, (ATerm)sort, 0);
  }

  ATabort("Entry has no Sort: %t\n", arg);
  return (ADTEntry)NULL;
}

/*}}}  */
/*{{{  ATbool ADThasEntryAlt(ADTEntry arg) */

ATbool ADThasEntryAlt(ADTEntry arg)
{
  if (ADTisEntryDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADTAlternative ADTgetEntryAlt(ADTEntry arg) */

ADTAlternative ADTgetEntryAlt(ADTEntry arg)
{
  
    return (ADTAlternative)ATelementAt((ATermList)arg, 1);
}

/*}}}  */
/*{{{  ADTEntry ADTsetEntryAlt(ADTEntry arg, ADTAlternative alt) */

ADTEntry ADTsetEntryAlt(ADTEntry arg, ADTAlternative alt)
{
  if (ADTisEntryDefault(arg)) {
    return (ADTEntry)ATreplace((ATermList)arg, (ATerm)alt, 1);
  }

  ATabort("Entry has no Alt: %t\n", arg);
  return (ADTEntry)NULL;
}

/*}}}  */
/*{{{  ATbool ADThasEntryPat(ADTEntry arg) */

ATbool ADThasEntryPat(ADTEntry arg)
{
  if (ADTisEntryDefault(arg)) {
    return ATtrue;
  }
  return ATfalse;
}

/*}}}  */
/*{{{  ADTPattern ADTgetEntryPat(ADTEntry arg) */

ADTPattern ADTgetEntryPat(ADTEntry arg)
{
  
    return (ADTPattern)ATelementAt((ATermList)arg, 2);
}

/*}}}  */
/*{{{  ADTEntry ADTsetEntryPat(ADTEntry arg, ADTPattern pat) */

ADTEntry ADTsetEntryPat(ADTEntry arg, ADTPattern pat)
{
  if (ADTisEntryDefault(arg)) {
    return (ADTEntry)ATreplace((ATermList)arg, (ATerm)pat, 2);
  }

  ATabort("Entry has no Pat: %t\n", arg);
  return (ADTEntry)NULL;
}

/*}}}  */

/*}}}  */
/*{{{  sort visitors */

/*{{{  ADTEntries ADTvisitEntries(ADTEntries arg, ADTEntry (*acceptHead)(ADTEntry)) */

ADTEntries ADTvisitEntries(ADTEntries arg, ADTEntry (*acceptHead)(ADTEntry))
{
  if (ADTisEntriesList(arg)) {
    return ADTmakeEntriesList(
        acceptHead ? acceptHead(ADTgetEntriesHead(arg)) : ADTgetEntriesHead(arg),
        ADTvisitEntries(ADTgetEntriesTail(arg), acceptHead));
  }
  ATabort("not a Entries: %t\n", arg);
  return (ADTEntries)NULL;
}

/*}}}  */
/*{{{  ADTEntry ADTvisitEntry(ADTEntry arg, ADTSort (*acceptSort)(ADTSort), ADTAlternative (*acceptAlt)(ADTAlternative), ADTPattern (*acceptPat)(ADTPattern)) */

ADTEntry ADTvisitEntry(ADTEntry arg, ADTSort (*acceptSort)(ADTSort), ADTAlternative (*acceptAlt)(ADTAlternative), ADTPattern (*acceptPat)(ADTPattern))
{
  if (ADTisEntryDefault(arg)) {
    return ADTmakeEntryDefault(
        acceptSort ? acceptSort(ADTgetEntrySort(arg)) : ADTgetEntrySort(arg),
        acceptAlt ? acceptAlt(ADTgetEntryAlt(arg)) : ADTgetEntryAlt(arg),
        acceptPat ? acceptPat(ADTgetEntryPat(arg)) : ADTgetEntryPat(arg));
  }
  ATabort("not a Entry: %t\n", arg);
  return (ADTEntry)NULL;
}

/*}}}  */

/*}}}  */
